const fs = require('fs')
const path = require('path')
const log = require('plugin-log')
module.exports = function generatepack (config) {
  const vuePath = path.resolve(path.resolve(__dirname, config.root), config.name + '.vue')
  if (fs.existsSync(vuePath)) {
    log('该文件已存在')
    return
  }
  const templateTask = [packageFilter(), packageAction(), packageTable(), packagePagination()]
  const scriptTask = [packageData(), packageLifecircle(), packageMethods()]
  // const styleTask = [packageStyle()]


  function packageStyle() {
    return `
<style lang='scss' rel='stylesheet/scss' scoped>
  .container{
    .c-selectwrap{
      margin: 20px 0;
      display: flex;
      justify-content: space-between;
      max-width: 1600px;
      .s-left{
        display: flex;
      }
    }
  }
</style>`
  }
  function packagePagination() {
    return `
    <el-pagination
      style='margin-top:30px;'
      :page-size='pageSize'
      @current-change='_handlePageChange'
      :current-page='pageNum'
      background
      layout="prev, pager, next"
      :total="total">
    </el-pagination>`
  }
  function packageTempTask() {
    let actionTemplate = ''
    templateTask.forEach(v => {
      actionTemplate += v
    })
    return `<template>
      <div class='container'>
        ${actionTemplate}
      </div>
    </template>
    `
  }
  function packageScriptTask() {
    let scriptTemplate = ''
    scriptTask.forEach(v => {
      scriptTemplate += v
    })
    return `<script>
      export default{
        ${scriptTemplate}
      }
    </script>
    `
  }
  function packageData() {
    let dataTemplate = ''
    config.filter.length && config.filter.forEach(v => {
      dataTemplate += `${v.keyword}: ${v.default == '' ?  '""' : v.default},`
    })
    for (let key in config.defaultData) {
      dataTemplate += `${key}: ${config.defaultData[key] == '' ? '""' : config.defaultData[key]},`
    }
    return `data(){
      return{
        ${dataTemplate}
      }
    },`
  }
  function packageMethods() {
    let methodsTemplate = ''
    config.action.length && config.action.forEach(v => {
      methodsTemplate += generateMethods(v)
    })
    return `methods: {
        _handlePageChange(e) {
          this.currentPage = e
          this.getData()
        },
        ${methodsTemplate}
    },`
  }
  function packageLifecircle() {
    return `
    created() {

    },
    mounted() {

    },
    `
  }
  function generateMethods(item) {
    let methodTemplate = ''
    switch(item) {
      case 'add': 
        methodTemplate = `_handleAdd() {

        }, \n `
        break;
      case 'reset': 
        let filterDefault = ''
        config.filter.length && config.filter.forEach(v => {
          filterDefault += `this.${v.keyword} = ${v.default == '' ? "''" : v.default} \n `
        })
        methodTemplate = `_handleReset() {
          ${filterDefault}
          this.currentPage = 1
          this._handleSearch()
        }, \n
        `
        break;
      case 'search': 
        let filterParams = ''
        config.filter.length && config.filter.forEach(v => {
          filterParams += `${v.keyword}: this.${v.keyword}, \n `
        })
        methodTemplate = `_handleSearch() {
          fetch.__INTERFACE__({
            pageNum: this.currentPage,
            pageSize: this.pageSize,
            ${filterParams}
          }).then(res => {
            this.tableData = res.list
            this.total = res.total
          }, e => {
            console.error(e)
          })
        }, \n `
        break;
    }
    return methodTemplate
  }
  function packageAction() {
    if (!config.action.length) return
    let actionTemplate = ''
    config.action.forEach(v => {
      actionTemplate += generateAction(v)
    })
    return `<div class='s-right'>${actionTemplate}</div>
    </div>`
  }
  function packageFilter() {
    if (!config.filter.length) return
    let filterTemplate = ''
    config.filter.forEach(v => {
      filterTemplate += generateFilter(v)
    })
    return `
    <div class='c-selectwrap'>
      <div class='s-left'>${filterTemplate}</div>`
  }
  function packageTable() {
    if (!config.tableData.list.length) return
    let tableTemplate = ''
    let list = config.tableData.list
    list.forEach(v => {
      tableTemplate += generateTable(v)
    })
    return `<el-table
    :data="${config.tableData.alies}"
    border
    style="width: 100%">${tableTemplate}</el-table>`
  }
  function generateTable(item) {
    let type = item.type || 'default'
    let tableItemTemplate = ''
    switch(type) {
      case 'default': 
        tableItemTemplate = ` <el-table-column
        prop="${item.fields}"
        label="${item.label}"
        width="180">
      </el-table-column>`
        break;
      case 'template':
          tableItemTemplate = ` <el-table-column
          label="${item.label}"
          width="180">
          <template slot-scope='scope'></template>
        </el-table-column>`
    }
    return tableItemTemplate.trim()
  }
  function generateAction(item) {
    let actionTemplate = ''
    switch(item) {
      case 'add': 
        actionTemplate = "<el-button @click='_handleAdd'>新建</el-button> \n "
        break;
      case 'reset': 
        actionTemplate = "<el-button @click='_handleReset'>重置</el-button> \n "
        break;
      case 'search': 
        actionTemplate = "<el-button @click='_handleSearch'>查询</el-button> \n "
        break;
    }
    console.log(item)
    return actionTemplate
  }
  function generateFilter(item) {
    let filterTemplate = ''
    switch(item.type) {
      case 'Input':
          filterTemplate = `<el-input placeholder="请输入${item.label}" style='margin-right:20px;' v-model='${item.keyword}'></el-input>`
          break;
      case 'Select':
          filterTemplate = 
          `<el-select v-model="${item.keyword}" placeholder="请选择${item.label}" style='margin-right:20px;'>
            <el-option
              v-for="item in ${item.keyword}Options"
              :key="item.value"
              :label="item.label"
              :value="item.value">
            </el-option>
          </el-select>`
          break;
      case 'DatePicker':
          filterTemplate =
           ` <el-date-picker
              v-model="${item.keyword}"
              type="daterange"
              range-separator="至"
              start-placeholder="开始日期"
              end-placeholder="结束日期">
            </el-date-picker>`
          break;
    }
    return filterTemplate.trim()
  }
//   let vueTemplate = 
// `<template>
//   <div class='container'> </div>
// </template>
// <script>
//   export default{
//     data() {
//       return {

//       }
//     },
//     mounted() {

//     },
//     methods: {

//     }
//   }
// </script>
// <style scoped>
// </style>
// `  
  log('vuepath' + vuePath)
  fs.writeFileSync(vuePath, packageTempTask() + packageScriptTask().trim() + packageStyle())
}
